﻿namespace DotNetUtility.ExtensionMethods
{
    using System;

    /// <summary>
    /// Different extensions method of Array.
    /// </summary>
    public static class ArrayExtensions
    {
        /// <summary>
        /// To clear all the elements value in the array.
        /// </summary>
        /// <param name="arrayToClear">The array to clear.</param>
        /// <returns>Cleared array.</returns>
        /// <example>
        ///     <code>
        ///         int[] myArray = { 1, 2, 3, 4, 5, 6, 7 };
        ///         Array array = myArray;
        ///         array = array.ClearAll();
        ///     </code>
        /// </example>
        /// Contributed by Mohammad Rahman
        public static Array ClearAll(this Array arrayToClear)
        {
            if (arrayToClear.IsNotEqualToNull())
                Array.Clear(arrayToClear, 0, arrayToClear.Length);
            return arrayToClear;
        }

        /// <summary>
        ///  To clear an array of type T.
        /// </summary>
        /// <typeparam name="T">The type of the array</typeparam>
        /// <param name="arrayToClear">The array to clear.</param>
        /// <returns>Cleared array.</returns>
        /// <example>
        ///     <code>
        ///         int[] myArray = { 1, 2, 3, 4, 5, 6, 7 };
        ///         int[] result = myArray.ClearAll<int>();
        ///     </code>
        /// </example>
        /// Contributed by Mohammad Rahman
        public static T[] ClearAll<T>(this T[] arrayToClear)
        {

            if (arrayToClear.IsNotEqualToNull())
                for (int i = arrayToClear.GetLowerBound(0); i <= arrayToClear.GetUpperBound(0); ++i)
                    arrayToClear[i] = default(T);
            return arrayToClear;
        }

        /// <summary>
        /// To clear a item at a specified position in the given array.
        /// </summary>
        /// <param name="arrayToClear">Array to clear.</param>
        /// <param name="at">The position of the item which to clear.</param>
        /// <returns>The cleared array.</returns>
        /// <example>
        ///     <code>
        ///         int[] myArray = { 1, 2, 3, 4, 5, 6, 7 };
        ///         Array array = myArray;
        ///         array = array.ClearAt(3);
        ///     </code>
        /// </example>
        /// Contributed by Mohammad Rahman
        public static Array ClearAt(this Array arrayToClear, int at)
        {
            if (arrayToClear.IsNotEqualToNull())
            {
                int arrayIndex = at.ToArrayInedx();
                if (arrayIndex.IsIndexInArray(arrayToClear))
                    Array.Clear(arrayToClear, arrayIndex, 1);
            }
            return arrayToClear;
        }

        /// <summary>
        /// To clear a item at a specified position in the given array.
        /// </summary>
        /// <typeparam name="T">The type of the array.</typeparam>
        /// <param name="arrayToClear">Array to clear</param>
        /// <param name="at">The position of the item which to clear.</param>
        /// <returns>The cleared array.</returns>
        /// <example>
        ///     <code>
        ///         int[] myArray = { 1, 2, 3, 4, 5, 6, 7 };
        ///         int[] result = myArray.ClearAt<int>(3);
        ///     </code>
        /// </example>
        /// Contributed by Mohammad Rahman
        public static T[] ClearAt<T>(this T[] arrayToClear, int at)
        {
            if (arrayToClear.IsNotEqualToNull())
            {
                int arrayIndex = at.ToArrayInedx();
                if (arrayIndex.IsIndexInArray(arrayToClear))
                    arrayToClear[arrayIndex] = default(T);
            }
            return arrayToClear;
        }

        /// <summary>
        /// To combine two same type arrray.
        /// </summary>
        /// <typeparam name="T">The type of the array.</typeparam>
        /// <param name="combineWith">The array to combine with.</param>
        /// <param name="arrayToCombine">The array to combine.</param>
        /// <returns>The combined array.</returns>
        /// <example>
        ///     <code>
        ///         int[] myArray1 = { 1, 2, 3, 4, 5, 6, 7 };
        ///         int[] myArray2 = { 1, 2, 3, 4, 5, 6, 7 };
        ///         int[] result = myArray1.CombineArray<int>(myArray2);
        ///     </code>
        /// </example>
        /// Contributed by Mohammad Rahman
        public static T[] CombineArray<T>(this T[] combineWith, T[] arrayToCombine)
        {
            if (combineWith != default(T[]) && arrayToCombine != default(T[]))
            {
                int initialSize = combineWith.Length;
                Array.Resize<T>(ref combineWith, initialSize + arrayToCombine.Length);
                Array.Copy(arrayToCombine, arrayToCombine.GetLowerBound(0), combineWith, initialSize, arrayToCombine.Length);
            }
            return combineWith;
        }
    }
}
